
def RVselect(synthPop,params):
  '''
   Limit the synthetic population to systems that would be
    detectable by radial velocity observations.
   The selection effect is prescribed by two parameters:
    RVcut - the minimum radial velocity amplitude (m/s)
    RVtime - the length of time for radial velocity observations (years)
  '''

  if not synthPop:
    return

#  print synthPop.keys()

  nplanets=len(synthPop['iplanet'])

  MEarthtoMSun=5.972e27/1.99e33
  Vkep1AU=2.97e4  # Earth's velocity is 30 km/s, given here in m/s (not cgs!)

  RVselectedPop={'Msini':[],'ap':[],'inc':[]}
  for i in range(nplanets):
# calculate the RV signal, in m/s
    massRatio=synthPop['Mp'][i]/synthPop['mstar'][i] *MEarthtoMSun
    V_keplerian=Vkep1AU * sqrt(synthPop['mstar'][i]/synthPop['ap'][i])
    RVsignal=massRatio * V_keplerian * sin(synthPop['inc'][i])

# reduce the effective signal if orbital period is longer than observing time
    period=sqrt(synthPop['ap'][i]**3/synthPop['mstar'][i])
    if period > params['RVtime']:
      RVsignal/=(period/params['RVtime'])**2.5

    if RVsignal>params['RVcut']:
      RVselectedPop['Msini'].append(synthPop['Msini'][i])
      RVselectedPop['ap'].append(synthPop['ap'][i])
      RVselectedPop['inc'].append(synthPop['inc'][i])

#      print 'RV signal, M,a,i',RVsignal, \
#            synthPop['Msini'][i],synthPop['ap'][i],synthPop['inc'][i]
            
  print
  print 'fraction of synthetic population detectable via RV: ', \
        len(RVselectedPop['Msini'])/float(len(synthPop['Msini']))
  
  return RVselectedPop
#______________________________________________________________________________

def RVdataSelect(allPlanets, verbose=False):
  '''
   Pick out the RV-detected systems from the overall set of real planets
  '''

#  print allPlanets.keys()

  RVdata={'semia':[],'msini':[]}

  nplanets=len(allPlanets['pl_name'])
#  print 'nplanets',nplanets,len(allPlanets['pl_msinie']), \
#        len(allPlanets['pl_orbsmax'])

  for i in range(nplanets):
    if allPlanets['pl_discmethod'][i]=='Radial Velocity':
      if not allPlanets['pl_orbsmax'][i]:        
        if verbose:
          print 'ERROR: semi-major axis is blank for RV planet', \
                allPlanets['pl_name'][i]
      elif not allPlanets['pl_msinie'][i]:
        if verbose:
          print 'ERROR: Msini is blank for RV planet', \
                allPlanets['pl_name'][i]
      else:      
        RVdata['semia'].append(allPlanets['pl_orbsmax'][i])
        RVdata['msini'].append(allPlanets['pl_msinie'][i])
    else:      
      if allPlanets['pl_msinie'][i]:
        if verbose:
          print 'non-RV-discovery with Msini value', \
                allPlanets['pl_name'][i], \
                allPlanets['kepler_name'][i]
      
  return RVdata
#______________________________________________________________________________

def transitSelect(synthPop,params):

  '''
   Simulate the selection effects of a transit survey.
   Limit the synthetic population to systems that would be
    detectable as transits.
   *** assumes that the systems already have a random inclination ***
   The selection effect is prescribed by two parameters:
    tranScale - the overall scaling
    tranTime - the timespan for the transit observations (years)
  '''

  if not synthPop:
    return

#  print synthPop.keys()

  RSuntoAU=7.e10/1.5e13
  REarthtoRSun=6.37e8/7.e10

  nplanets=len(synthPop['iplanet'])

  transitSelectedPop={'Mp':[],'Rp':[],'ap':[],'inc':[]}
  for i in range(nplanets):

    mstar=synthPop['mstar'][i]
# assume a linear relationship for Rstar and Mstar
#  this is about right for 0.1-1.0 Mstar
    rstar=mstar
    inc=synthPop['inc'][i]

    rstaroa=rstar/synthPop['ap'][i] *RSuntoAU
#    if (pi/2. - inc) < rstaroa:
#      print 'TRANSIT!!',synthPop['ap'][i],rstaroa

    transitSignal=(synthPop['Rp'][i]/rstar *REarthtoRSun)**2
# convert to parts-per-million
    transitSignal*=1.e6
#    print 'transitSignal',transitSignal,params['transitCut']
     
# reduce the transit likelihood if orbital period is longer than observing time
# make the signal a function of the number of transits
    period=sqrt(synthPop['ap'][i]**3/synthPop['mstar'][i])
    nTransits=params['tranTime']/period

# semi-arbitrary threshold for transit detection
#  - require at least 3 transits
#  - include some compensating factor for # of transits
#     for simplicity, just assume that the detection rate
#     drops off as sqrt(# transits), if less than 25
    if nTransits>3:
      transitSignal*=min(1.,sqrt(nTransits/25.))

      if transitSignal>params['transitCut']:
    
        if (pi/2. - inc) < rstaroa * params['tranScale']:
          transitSelectedPop['Mp'].append(synthPop['Mp'][i])
          transitSelectedPop['Rp'].append(synthPop['Rp'][i])
          transitSelectedPop['ap'].append(synthPop['ap'][i])
          transitSelectedPop['inc'].append(synthPop['inc'][i])

#        print 'transit signal, M,a,i',transitSignal, \
#          synthPop['Msini'][i],synthPop['ap'][i],synthPop['inc'][i]
            
  print
  print 'fraction of synthetic population detectable via transit: ', \
        len(transitSelectedPop['Mp'])/float(len(synthPop['Mp']))
  
  return transitSelectedPop
#______________________________________________________________________________

def keplerSelect(synthPop, verbose=False):
  '''
   Pick out the Kepler systems from the overall set of real planets
  '''

#  print allPlanets.keys()

  keplerdata={'semia':[],'Rp':[],'Mp':[]}

  nplanets=len(allPlanets['pl_name'])
#  print 'nplanets',nplanets,len(allPlanets['pl_msinie']), \
#        len(allPlanets['pl_orbsmax'])

  for i in range(nplanets):
    include=0
    if allPlanets['pl_discmethod'][i]=='Pulsation Timing Variations' or \
       allPlanets['pl_discmethod'][i]=='Transit Timing Variations' or \
       allPlanets['pl_discmethod'][i]=='Orbital Brightness Modulation' or \
       allPlanets['pl_discmethod'][i]=='Eclipse Timing Variations' or \
       allPlanets['pl_discmethod'][i]=='Microlensing' or \
       allPlanets['pl_discmethod'][i]=='Pulsar Timing' or \
       allPlanets['pl_discmethod'][i]=='Radial Velocity' or \
       allPlanets['pl_discmethod'][i]=='Astrometry' or \
       allPlanets['pl_discmethod'][i]=='Imaging':
      pass
    elif allPlanets['pl_discmethod'][i]=='Transit':

# actually this is currently set to all Kepler detections
# if you want only Kepler discoveries (i.e. not TrES-2, HAT-P-7,11)
#  then comment out the second line of this conditional
      if 'epler' in allPlanets['pl_name'][i] or \
             'Kepler' in allPlanets['kepler_name'][i] or \
             'KOI' in allPlanets['pl_name'][i] or \
             'PH' in allPlanets['pl_name'][i]:  # PH=Planet Hunters
        include=1
#      else: 
#        print 'non-Kepler transit:',allPlanets['pl_name'][i], \
#              allPlanets['kepler_name'][i]

    elif allPlanets['pl_discmethod'][i]=='' and \
         allPlanets['koi_disposition'][i]=='CANDIDATE':
      include=1

    else:
      print 'strange else:',allPlanets['pl_discmethod'][i], \
            allPlanets['koi_disposition'][i]

    if include:
      if allPlanets['pl_rade'][i]:
        if allPlanets['pl_orbsmax'][i]:
          keplerdata['semia'].append(allPlanets['pl_orbsmax'][i])
          keplerdata['Rp'].append(allPlanets['pl_rade'][i])
          convertedMass=TraubPlanetMass(allPlanets['pl_rade'][i])
          keplerdata['Mp'].append(convertedMass)

        else:
          if verbose:
            print 'NOTE: Kepler planet without semi-major axis:', \
                  allPlanets['kepid'][i], \
                  allPlanets['pl_name'][i],allPlanets['kepler_name'][i]
      else:
        if verbose:
          print 'NOTE: Kepler planet without defined radius:', \
                allPlanets['kepid'][i], \
                allPlanets['pl_name'][i],allPlanets['kepler_name'][i]
            

#  exit()
  return keplerdata
#______________________________________________________________________________

def onlyConfirmed(allPlanets):

  confirmedPlanets={}
  for key in allPlanets:
    confirmedPlanets[key]=[]
    
  for i in range(len(allPlanets['pl_name'])):
    if allPlanets['pl_name'][i]:
      for key in allPlanets:
        confirmedPlanets[key].append(allPlanets[key][i])

      if allPlanets['koi_disposition'][i]=='CANDIDATE':
        print 'ERROR: candidate with confirmed name', \
              allPlanets['pl_name'][i]
    else:
      if allPlanets['koi_disposition'][i]<>'CANDIDATE':
        print 'ERROR: confirmed with no name?', \
              allPlanets['koi_disposition'][i]
  return confirmedPlanets
#______________________________________________________________________________

def addRandomInc(synthPop):
  '''
   Add random inclinations to the synthetic population.
  '''

  if not synthPop:
    return
  
  nplanets=len(synthPop['iplanet'])

# give a random inclination to each planet in the synthetic population
  synthPop['inc']=['']*nplanets

# inc=0 means face-on; inc=90 degrees means edge-on
# edge-on is more likely.  median inclination should be 60 degrees

  for i in range(nplanets):
#    inc=rand() * pi/2.
#    print 'random inclination',inc

    cosinc=random.random()
    inc=math.acos(cosinc)
    synthPop['inc'][i]=inc
    
  return synthPop
#______________________________________________________________________________
#______________________________________________________________________________

def addMsinI(synthPop):
  '''
   Calculate M.sin(i) for each planet
  '''

  if not synthPop:
    return
  
  nplanets=len(synthPop['iplanet'])

  if not synthPop['inc']:
    exit('random planet inclinations not set yet')
  
# Msini = M * sin(i)
  synthPop['Msini']=array(synthPop['Mp'])*sin(array(synthPop['inc']))

  av=0
  count=0.
  for i in range(nplanets):
    if synthPop['Mp'][i]>0.:
#      print 'M / Msini',synthPop['Mp'][i]/synthPop['Msini'][i], \
#            synthPop['inc'][i]
#            synthPop['Msini'][i],synthPop['Mp'][i], \
      av+=synthPop['Mp'][i]/synthPop['Msini'][i]
      count+=1
#  print 'number of planets with defined mass',count,nplanets      
#  print 'average M/Msini in synthetic pop',av/count  /(pi/2)

  return synthPop
#______________________________________________________________________________

def addRp(synthPop):
  if synthPop:
    synthPop['Rp']=[]
    for i in range(len(synthPop['Mp'])):
      convertedRadius=TraubPlanetRadius(synthPop['Mp'][i])
      synthPop['Rp'].append(convertedRadius)
  return synthPop
#______________________________________________________________________________

def checkDiff(observations,model,varName):
  if synthPop:
#    print 'comparing two arrays',len(observations),len(model)

    obsValues=selectValues(observations)
      
    D,p=scipy.stats.ks_2samp(obsValues,array(model))
                               
    print ' K-S test p-value:',p,' ('+varName+')'
  return
#______________________________________________________________________________

def selectValues(inarray):
  '''
  Pick out the values of the array that are defined.
  '''  
  out=[]
  for i in range(len(inarray)):
    if not isinstance(inarray[i],float):
      pass
    else:
      out.append(inarray[i])

  return numpy.array(out)
#______________________________________________________________________________

